/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.allgemein;

/**
 * Die möglichen Zustände einer Störfallsituation.
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public enum StoerfallZustand {

	/** . */
	STOERUNG(0, "Störung"),
	/** . */
	KEINE_AUSSAGE(1, "keine Aussage"),
	/** . */
	FREIER_VERKEHR(2, "freier Verkehr"),
	/** . */
	LEBHAFTER_VERKEHR(3, "lebhafter Verkehr"),
	/** . */
	DICHTER_VERKEHR(4, "dichter Verkehr"),
	/** . */
	ZAEHFLIESSENDER_VERKEHR(5, "zähfließender Verkehr"),
	/** . */
	STOCKENDER_VERKEHR(6, "stockender Verkehr"),
	/** . */
	STAU(7, "Stau");

	/**
	 * Numerische Darstellung der Störfallsituation.
	 */
	public final int id;

	/**
	 * Text-Darstellung der Störfallsituation.
	 */
	public final String text;

	@Override
	public String toString() {
		return text;
	}

	/**
	 * Gibt den zur übergebenen Zustands-Id gehörende StörfallZustand zurück.
	 * 
	 * @param id
	 *            Störfallzustands-Id
	 * @return der zur übergebenen Zustands-Id gehörende StörfallZustand oder
	 *         null, falls es zur Id keinen StörfallZustand gibt
	 */
	public static StoerfallZustand get(final int id) {
		for (StoerfallZustand zustand : values()) {
			if (zustand.id == id) {
				return zustand;
			}
		}
		return null;
	}

	/**
	 * Privater Konstruktor.
	 * 
	 * @param id
	 *            ID
	 * @param text
	 *            Text
	 */
	StoerfallZustand(final int id, final String text) {
		this.id = id;
		this.text = text;
	}
}
