/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.konfiguration;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es werden gemeinsame Eigenschaften von Dav-Stellvertreter-Objekten
 * modelliert.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class DavProxyObjekt {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private final static Debug _debug = Debug.getLogger();

	/**
	 * das zugeordnete Dav-Objekt.
	 */
	private SystemObject _davObjekt;

	/**
	 * Erstellt eine neue Instanz vom Typ <code>DavProxyObjekt</code>.
	 * 
	 * @param davObjekt
	 *            das zuzuordnente Dav-Objekt
	 */
	protected DavProxyObjekt(SystemObject davObjekt) {
		_davObjekt = davObjekt;
	}

	/**
	 * @return das zugeordnete Dav-Objekt
	 */
	public SystemObject getDavObjekt() {
		return _davObjekt;
	}

	@Override
	public String toString() {
		return getDavObjekt().getPid();
	}

}
