/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.konfiguration;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein StrassenAbschnitt modelliert.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class StrassenAbschnitt extends DavProxyObjekt {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug _debug = Debug.getLogger();

	/**
	 * Messquerschnitt, an dem dieser Strassenabschnitt anfängt.
	 */
	private MessQuerschnitt _vonMq;

	/**
	 * Messquerschnitt, an dem dieser Strassenabschnitt endet.
	 */
	private MessQuerschnitt _nachMq;

	/**
	 * Erstellt eine neue Instanz vom Typ <code>StrassenAbschnitt</code>.
	 * 
	 * @param davObjekt
	 *            das zuzuordnente Dav-Objekt
	 * @param vonMq
	 *            Messquerschnitt, an dem dieser Strassenabschnitt anfängt
	 * @param nachMq
	 *            Messquerschnitt, an dem dieser Strassenabschnitt endet
	 * 
	 */
	public StrassenAbschnitt(SystemObject davObjekt, MessQuerschnitt vonMq, MessQuerschnitt nachMq) {
		super(davObjekt);
		_vonMq = vonMq;
		_nachMq = nachMq;
	}

	/**
	 * @return den Messquerschnitt, an dem dieser Strassenabschnitt anfängt
	 */
	public MessQuerschnitt getVonMq() {
		return _vonMq;
	}

	/**
	 * @return den Messquerschnitt, an dem dieser Strassenabschnitt endet
	 */
	public MessQuerschnitt getNachMq() {
		return _nachMq;
	}

}
