/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.meldungserstellung;

import de.bsvrz.ste.tmc.allgemein.RDSEreignis;
import de.bsvrz.ste.tmc.konfiguration.StrassenSegment;
import de.bsvrz.ste.tmc.verkehrsanalyse.Stoerfall;

/**
 * Ereignisdaten.
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class EreignisDaten {
	public EreignisDaten(long startSegmentOffset, final long endSegmentOffset, final StrassenSegment startSegment,
			StrassenSegment endSegment, RDSEreignis ereignis, long start, long ende, Stoerfall lokal) {
		this.startSegmentOffset = startSegmentOffset;
		this.endSegmentOffset = endSegmentOffset;
		this.startSegment = startSegment;
		this.endSegment = endSegment;
		this.ereignis = ereignis;
		this.start = start;
		this.ende = ende;
		this.lokal = lokal;
	}

	public EreignisDaten(final EreignisDaten daten) {
		this(daten.startSegmentOffset, daten.endSegmentOffset, daten.startSegment, daten.endSegment, daten.ereignis,
				daten.start, daten.ende, daten.lokal);
	}

	public boolean equals(Object obj) {
		if (obj instanceof EreignisDaten) {
			EreignisDaten daten = (EreignisDaten) obj;
			return (startSegmentOffset == daten.startSegmentOffset && endSegmentOffset == daten.startSegmentOffset
					&& startSegment == daten.startSegment && endSegment == daten.endSegment
					&& ereignis == daten.ereignis && start == daten.start && ende == daten.ende
					&& lokal == daten.lokal);
		} else {
			return false;
		}
	}

	public final long startSegmentOffset;

	public final long endSegmentOffset;

	public final StrassenSegment startSegment;

	public final StrassenSegment endSegment;

	public final RDSEreignis ereignis;

	public final long start;

	public final long ende;

	public final Stoerfall lokal;

	public long getLaenge() {
		return endSegmentOffset + ende - (startSegmentOffset + start);
	}

	@Override
	public String toString() {
		return "EreignisDaten {Ereignis: " + ereignis + ": Länge " + getLaenge() + "m beginnend auf "
				+ startSegment.getDavObjekt().getPid() + " bei " + start + "m " + " (startSegmentOffset: "
				+ startSegmentOffset + ") endet auf " + endSegment.getDavObjekt().getPid() + " bei " + ende + "m "
				+ " (endSegmentOffset: " + endSegmentOffset + ")}";
	}
}
