/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.meldungserstellung;

/**
 * Es wird festgelegt, was beim Starten der Meldungserstellung publiziert
 * wird:<br>
 * - Nur aktuelle Änderungen der Meldungen - falls vorhanden,<br>
 * - Alles, was vorher (beim letzten Programmlauf) auch publiziert war.<br>
 *
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public enum PublikationInitial {
	/** Beim Initialisieren werden nur Änderungen publiziert. */
	AENDERUNGEN(0),

	/** Beim Initialisieren wird der vorherige Zustand wiederhergestellt. */
	ALLES(1);

	/**
	 * Numerische Darstellung der Location-Methode.
	 */
	public final int id;

	/**
	 * Gibt die zur übergebenen Id gehörende PublikationInitial-Instanz zurück.
	 * 
	 * @param id
	 *            Id für die initiale Publikation
	 * @return die zur übergebenen Id gehörende PublikationInitial-Instanz oder
	 *         null, falls es zur Id keine Instanz gibt
	 */
	public static PublikationInitial get(final int id) {
		for (PublikationInitial publikation : values()) {
			if (publikation.id == id) {
				return publikation;
			}
		}
		return null;
	}

	/**
	 * Konstruktor.
	 * 
	 * @param id
	 *            ID
	 */
	PublikationInitial(final int id) {
		this.id = id;
	}

}
