/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.tmc.allgemein.Beschleunigung;
import de.bsvrz.ste.tmc.allgemein.StoerfallZustand;
import de.bsvrz.ste.tmc.meldungserstellung.VerwaltungMeldungen;
import de.bsvrz.sys.funclib.debug.Debug;

public class PrueffallBasis {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	private final String LMS_TYP = "typ.tmcLandesMeldeStelle";

	public final String MODELL_ATG = "atg.tmcModell";

	/**
	 * Die DAV-Verbindung.
	 */
	private ClientDavInterface connection;

	/**
	 * @return die DAV-Verbindung
	 */
	protected ClientDavInterface getConnection() {
		return connection;
	}

	/**
	 * Die Prueffall-Umgebung.
	 */
	PrueffallUmgebung _umgebung;

	/**
	 * @return die Prueffall-Umgebung
	 */
	protected PrueffallUmgebung getUmgebung() {
		return _umgebung;
	}

	protected PrueffallBasis(ClientDavInterface connection) {
		this.connection = connection;

		_umgebung = new PrueffallUmgebung(connection);
	}

	protected void setzteIndikatoren(int[] pZustaende) {
		setzteIndikatoren(pZustaende, new int[0]);
	}

	protected void setzteIndikatoren(int[] pZustaende, int[] pGueten) {
		int[] zustaende = new int[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int laengeZ = pZustaende.length > zustaende.length ? zustaende.length : pZustaende.length;

		int gueten[] = new int[] { 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
				100 };
		int laengeG = pGueten.length > gueten.length ? gueten.length : pGueten.length;

		for (int i = 0; i < laengeZ; i += 1) {
			zustaende[i] = pZustaende[i];
		}

		for (int i = 0; i < laengeG; i += 1) {
			gueten[i] = pGueten[i];
		}

		int[] index1 = new int[] { 1, 2, 3, 3, 1, 2, 3, 3, 1, 2, 3, 1, 2, 3, 3, 1, 2, 3 };
		int[] index2 = new int[] { 1, 1, 1, 2, 2, 2, 3, 4, 3, 3, 5, 4, 4, 6, 7, 5, 5, 8 };

		StoerfallIndikator[] indikatoren = new StoerfallIndikator[] { getUmgebung().getIndikatorVerfahren1()[0],
				getUmgebung().getIndikatorVerfahren2()[0], getUmgebung().getIndikatorVerfahren3()[0],
				getUmgebung().getIndikatorVerfahren3()[1], getUmgebung().getIndikatorVerfahren1()[1],
				getUmgebung().getIndikatorVerfahren2()[1], getUmgebung().getIndikatorVerfahren3()[2],
				getUmgebung().getIndikatorVerfahren3()[3], getUmgebung().getIndikatorVerfahren1()[2],
				getUmgebung().getIndikatorVerfahren2()[2], getUmgebung().getIndikatorVerfahren3()[4],
				getUmgebung().getIndikatorVerfahren1()[3], getUmgebung().getIndikatorVerfahren2()[3],
				getUmgebung().getIndikatorVerfahren3()[5], getUmgebung().getIndikatorVerfahren3()[6],
				getUmgebung().getIndikatorVerfahren1()[4], getUmgebung().getIndikatorVerfahren2()[4],
				getUmgebung().getIndikatorVerfahren3()[7] };

		String text = "";

		for (int i = 0; i < zustaende.length; i += 1) {
			if (zustaende[i] > 0 && zustaende[i] <= 6) {
				int zustand = zustaende[i] == 6 ? 7 : zustaende[i];
				float guete = ((float) gueten[i]) / 100;
				guete = guete < 0.0f ? 0.0f : guete > 1.0f ? 1.0f : guete;
				indikatoren[i].send(zustand, guete);
				text += (!text.equals("") ? "\n" : "") + "Störfallindikator S" + index1[i] + "," + index2[i]
						+ " wird mit Zustand \"" + zustaende[i] + "\" (" + StoerfallZustand.get(zustand)
						+ (gueten[i] != 100 ? ", Güte " + guete : "") + ") belegt";
			}
		}
		if (!text.equals("")) {
			DEBUG.info(text);
		}
	}

	public void setzteInitialParameter() {
		final AttributeGroup atg = getConnection().getDataModel().getAttributeGroup(MODELL_ATG);
		Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");

		final Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(),
				new DataDescription(atg, asp)).getData();
		if (altParam != null) {
			setOriginalParameter(altParam);

			final Data neuParam = altParam.createModifiableCopy();
			String liste = null;
			String text = "Parameteränderungen:";

			ReferenceArray refArray = neuParam.getItem("RDSSIVerfahren").asReferenceArray();

			Aspect[] verfahren = _umgebung.get_verfahren();
			// String verfahren[] = {"asp.störfallVerfahrenStandard",
			// "asp.störfallVerfahrenRDS", "asp.störfallVerfahrenFuzzy"};
			// text += "\n\tRDSSIVerfahren: ";

			refArray.setLength(verfahren.length);

			for (int i = 0; i < verfahren.length; i++) {
				// Aspect aspect =
				// getConnection().getDataModel().getAspect(verfahren[i]);
				refArray.getReferenceValue(i).setSystemObject(verfahren[i]);
				text += verfahren[i].getName() + " ";
			}

			neuParam.getItem("RDSBerücksichtigeStauverlaufsanalyse").asTextValue().setText("Nein");
			neuParam.getItem("RDSPlausibStauverlaufsanalyse").asTextValue().setText("Nein");
			neuParam.getItem("RDSLSIAusdehnung").asUnscaledValue().set(200); // GIESELER:
																				// Angabe
																				// FEHLT
																				// in
																				// PrSpez
			neuParam.getItem("RDSDMax").asUnscaledValue().set(1500);
			neuParam.getItem("RDSDStoerfallAnfang").asUnscaledValue().set(500);
			neuParam.getItem("RDSDStoerfallEnde").asUnscaledValue().set(500);
			neuParam.getItem("RDSMindestLaenge").asUnscaledValue().set(1000);
			neuParam.getItem("RDSDMaxEinzelStoerfall").asUnscaledValue().set(2000);
			neuParam.getItem("RDSNichtAddieren").asTextValue().setText("Nein");
			neuParam.getItem("RDSBaustelleSuchbereichVor").asUnscaledValue().set(500); // ??
			neuParam.getItem("RDSBaustelleSuchbereichNach").asUnscaledValue().set(500); // ??
			neuParam.getItem("RDSMaxOhneDaten").asUnscaledValue().set(0); // ??

			final Array dataArray = neuParam.getItem("RDSAusdehnungProLSI").asArray();
			dataArray.setLength(0);

			asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
			new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam)
					.start();
			// _debug.config("Parameter-Aktualisierung gestartet");
			// _debug.info(text);
		}
	}

	private Data _originalParameter = null;

	protected void setOriginalParameter(Data originalParameter) {
		_originalParameter = originalParameter;
	}

	protected Data getOriginalParameter() {
		return _originalParameter;
	}

	protected void parameterRestaurieren() {
		if (getOriginalParameter() != null) {
			AttributeGroup atg = connection.getDataModel().getAttributeGroup(MODELL_ATG);
			Aspect asp = connection.getDataModel().getAspect("asp.parameterVorgabe");

			new ParameterSender(connection, getUmgebung().getLms(), new DataDescription(atg, asp),
					getOriginalParameter()).start();
			DEBUG.config("Parameter-Aktualisierung gestartet");
			DEBUG.info("Parameter " + atg.getName() + " werden auf ihre Originalwerte zurückgesetzt");
		}
	}

	protected void warte(long warteZeit) {
		connection.sleep(warteZeit / Beschleunigung.WERT);
	}

	protected void warteAufEingabe() {
		System.out.print("w für weiter oder e zum beenden ");
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String inputLine = "";
		while (true) {
			try {
				inputLine = in.readLine();
			} catch (IOException e) {
				DEBUG.error("Fehler", e);
			}
			if (inputLine != null) {
				if (inputLine.startsWith("e") || inputLine.startsWith("E")) {
					System.exit(0);
				}
				if (inputLine.startsWith("w") || inputLine.startsWith("W")) {
					return;
				}
			}
		}
	}

	protected void bereinigeRdsMeldungen() {
		ConfigurationObject lms = (ConfigurationObject) getUmgebung().getLms();
		if (lms == null) {
			throw new IllegalArgumentException(
					"Objekt mit PID '" + PrueffallUmgebung.LANDESMELDESTELLE_PID + "' existiert nicht");
		}

		if (!lms.isOfType(LMS_TYP)) {
			throw new IllegalArgumentException(
					"Objekt mit PID '" + lms.getPid() + "' ist nicht vom Typ '" + LMS_TYP + "'");
		}

		MutableSet menge = lms.getMutableSet(VerwaltungMeldungen.MENGE);
		// List<SystemObject> liste = menge.getElements();

		List<SystemObject> liste = connection.getDataModel().getType("typ.tmcVerkehrsMeldung").getElements();

		if (liste.isEmpty()) {
			return;
		}
		int n = 0;
		for (SystemObject meldung : liste) {
			if (meldung.getPid().startsWith(VerwaltungMeldungen.PID_PREFIX)) {
				try {
					menge.remove(meldung);
					((DynamicObject) meldung).invalidate();
				} catch (ConfigurationChangeException e) {
					throw new RuntimeException("Fehler beim bereinigen von Objekt " + meldung.getPid(), e);
				}
				n += 1;
			}
		}
		DEBUG.info(n + " RDS-Meldung(en) von " + lms.getName() + " (deren PID mit \"" + VerwaltungMeldungen.PID_PREFIX
				+ "\" beginnt) wurde(n) gelöscht");
	}

	protected ConfigurationArea getDefaultConfigurationArea() {
		ConfigurationArea ca = null;
		DataModel dm = connection.getDataModel();
		ConfigurationObject co = dm.getConfigurationAuthority();
		AttributeGroup atg = dm.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
		Data data = co.getConfigurationData(atg);
		if (data != null) {
			TextArray array = data.getTextArray("defaultBereich");
			if (array.getLength() > 0) {
				ca = (ConfigurationArea) dm.getObject(array.getText(0));
			}
		}
		if (ca == null) {
			throw new IllegalArgumentException("Es konnte kein Default-Konfigurationsbereich bestimmt werden");
		}
		return ca;
	}

	private Process applikationsProzess;

	protected Process getApplikationsProzess() {
		return applikationsProzess;
	}

	protected void starteApplikationsProzess() throws IOException {
		final String separator = "/"; // System.getProperty("file.separator");
		final String javaHome = System.getProperty("java.home");
		final String classPath = "\"" + System.getProperty("java.class.path") + "\"";

		final ClientDavParameters params = connection.getClientDavParameters();
		final String datenverteiler = params.getDavCommunicationAddress() + ":"
				+ params.getDavCommunicationSubAddress();
		final String benutzer = params.getUserName();

		String cmdLine = javaHome + separator + "bin" + separator + "java";
		cmdLine += " -cp " + classPath + " -Xmx1500m ";
		cmdLine += " -Dfile.encoding=UTF-8 ";

		cmdLine += "de.bsvrz.ste.tmc.TMCApp";
		cmdLine += " -datenverteiler=" + datenverteiler;
		cmdLine += " -benutzer=" + benutzer;
		cmdLine += " -authentifizierung=" + "passwd"; // Passwortdatei im
														// Arbeitsverzeichnis!
														// (in Eclipse
														// Projektverz.)
		cmdLine += " -netz=" + getUmgebung().getVmn().getPid();
		cmdLine += " -lms=" + getUmgebung().getLms().getPid();
		cmdLine += " -idOrganisation=" + "\"D BW LMS-BW\"";
		cmdLine += " -idNummer=" + "65450";
		cmdLine += " -ablaufOffset=" + "240";

		cmdLine += " -debugLevelStdErrText=" + "INFO";

		// cmdLine += " -publikationInitial=1";

		applikationsProzess = Runtime.getRuntime().exec(cmdLine);
		new StdErrCatcher().start();
	}

	private class StdErrCatcher extends Thread {
		private byte[] buffer = new byte[1024];

		@Override
		public void run() {
			while (true) {
				try {
					if (getApplikationsProzess().getErrorStream().available() != 0) {
						final int anzahl = getApplikationsProzess().getErrorStream().read(buffer);

						System.out.print("" + new String(buffer, 0, anzahl));
					} else {
						sleep(100);
					}
				} catch (final IOException e) {
					break;
				} catch (final InterruptedException e) {
					// TODO Automatisch erstellter Catch-Block
					e.printStackTrace(System.out);
				}
			}
		}
	}
}
